/****************************************************************************

    This file is part of The Newtonator soft synth.

    Copyright (C) 2010-2011 Michael Bechard <mbechard@users.sourceforge.net>

    The Newtonator is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    The Newtonator is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with The Newtonator.  If not, see <http://www.gnu.org/licenses/>.

****************************************************************************/

#ifndef NEWTPARENTWIDGET_H
#define NEWTPARENTWIDGET_H

#include <gtkmm.h>
#include <VariSource.h>
#include <INewtSettings.h>
#include "newt_scalar_widget.h"
#include "control_enum.h"
#include "sensitivity_container.h"
#include "selection_widget.h"

class NewtParentWidget : public Gtk::Notebook
{
    public:

    NewtParentWidget();
    virtual ~NewtParentWidget();
    inline bool GetHideValueChange() const { return m_hideValueChange; }

    //one signal to fire when any control has changed value
    typedef sigc::signal<void, ControlEnum, stk::StkFloat> type_signal_value_change;
    inline type_signal_value_change signal_value_has_changed() { return m_valueHasChanged; }

    //one signal to fire when a control needs to change
    inline type_signal_value_change signal_value_should_change() { return m_valueShouldChange; }

    void EmitValueChange(ControlEnum ctrlID, stk::StkFloat val);
    void ChangeValue(ControlEnum ctrl, const stk::StkFloat val);
    void on_value_should_change(ControlEnum ctrl, stk::StkFloat val);

    protected:

    Glib::RefPtr<Gtk::ListStore> m_refWFTreeModel;
    Glib::RefPtr<Gtk::ListStore> m_refCBTreeModel;
    Gtk::Adjustment m_divAdjust1;
    Gtk::Adjustment m_divAdjust2;
    type_signal_value_change m_valueHasChanged;
    type_signal_value_change m_valueShouldChange;
    bool m_hideValueChange;

    void SetGModFreqFollow(bool val);
    void InitWFComboBox(SelectionWidget<VariSource::Type>& comboBox);
    void InitCBComboBox();
    void InitModulatorFreqs();

    //events
    void on_ceilingBehavior_changed();
    void on_gmodEnv_toggled();
    void on_gmodFollowFreq_toggled();
    void on_stereo_toggled();
    void on_gmodFreqDiv_value_changed();
    void on_wobbleScale_value_changed();
    void on_gmodDepth_value_changed();

    private:

    //Page 1
    Gtk::Table m_tab1a;
    Gtk::Table m_tab1b;
    Gtk::Table m_tab1c;
    Gtk::Table m_tab1d;
    Gtk::Table m_tab1e;
    Gtk::Frame m_frame1a;
    Gtk::Frame m_frame1b;
    Gtk::Frame m_frame1c;
    Gtk::Frame m_frame1d;
    NewtScalarWidget m_mainEnvAtk;
    NewtScalarWidget m_mainEnvDec;
    NewtScalarWidget m_mainEnvSus;
    NewtScalarWidget m_mainEnvRel;
    NewtScalarWidget m_velocReset;
    NewtScalarWidget m_velocDelay;
    NewtScalarWidget m_ceilingBounce;
    NewtScalarWidget m_wobbleA;
    NewtScalarWidget m_wobbleB;
    NewtScalarWidget m_wobbleScale;
    NewtScalarWidget m_wobbleTimbre;
    NewtScalarWidget m_velocRM;
    NewtScalarWidget m_velocSaH;
    NewtScalarWidget m_amp;
    SelectionWidget<CeilingBehavEnum> m_ceilingBehavior;
    Gtk::CheckButton m_stereo;
    SensitivityContainer m_wobbleSensitivity;

    //Page 2
    Gtk::Table m_tab2a;
    Gtk::Table m_tab2b;
    Gtk::Table m_tab2c;
    Gtk::Table m_tab2d;
    Gtk::Frame m_frame2a;
    Gtk::Frame m_frame2b;
    Gtk::Frame m_frame2c;
    NewtScalarWidget m_gmodEnvAtk;
    NewtScalarWidget m_gmodEnvDec;
    NewtScalarWidget m_gmodEnvSus;
    NewtScalarWidget m_gmodEnvRel;
    NewtScalarWidget m_gravScale;
    NewtScalarWidget m_gmodDepth;
    NewtScalarWidget m_gmodDCOffset;
    NewtScalarWidget m_gmodFreqAbs;
    NewtScalarWidget m_gmodFreqDrift;
    NewtScalarWidget m_gmodDeadZone;
    Gtk::CheckButton m_gmodEnv;
    Gtk::CheckButton m_gmodFollowFreq;
    SelectionWidget<VariSource::Type> m_gmodWave;
    Gtk::HScale m_gmodFreqDiv;
    Gtk::Label m_gmodFreqDivLab;
    SensitivityContainer m_gmodEnvSensitivity;
    SensitivityContainer m_gmodEnvFollowYesSensitivity;
    SensitivityContainer m_gmodEnvFollowNoSensitivity;
    SensitivityContainer m_gmodDepthSensitivity;
};

#endif // NEWTPARENTWIDGET_H
